<?php

namespace FiloBlu\Newsletter\Model\Plugin;

class Subscriber {

    protected $_objectManager;
    protected $request;
    protected $scopeConfig;

    /**
     * Helper
     *
     * @var \FiloBlu\Newsletter\Helper\Data
     */
    public $_helper;
    protected $_subscriberInfoFactory;

    /**
     * @var \Magento\Framework\App\Response\Http
     */
    protected $response;

    public function __construct(
    \Magento\Framework\ObjectManagerInterface $objectManager, \Magento\Framework\App\Request\Http $request, \FiloBlu\Newsletter\Helper\Data $helper, \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory, \Magento\Framework\App\Response\Http $response, \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig) {
        $this->_objectManager = $objectManager;
        $this->request = $request;
        $this->_helper = $helper;
        $this->_subscriberInfoFactory = $subscriberInfoFactory;
        $this->response = $response;
        $this->scopeConfig = $scopeConfig;
    }

    public function afterSubscribe(
    $subscriber, $email
    ) {
        if ($this->_helper->getConfig("popup/enabled")) {
            if (!$this->_helper->getPopupCookie()) {
                $this->_helper->setPopupCookie();
            }
        }

        if ($this->_helper->getConfig("general/enabled")) {

            $fieldsMapping = unserialize($this->_helper->getConfig('general/fields_mapping'));

            if ($fieldsMapping && count($fieldsMapping)) {
                $post = $this->request->getPost();

                $model = $this->_subscriberInfoFactory->create();
                $model->setSubscriberId($subscriber->getSubscriberId());

                foreach ($fieldsMapping as $field) {
                    if (isset($post[$field["postname"]])) {
                        if (is_array($post[$field["postname"]])) {
                            $valStr = "";
                            foreach ($post[$field["postname"]] as $val) {
                                if (is_string($val)) {
                                    if ($valStr) {
                                        $valStr .= ' | ' . $val;
                                    } else {
                                        $valStr = $val;
                                    }
                                }
                            }

                            if ($valStr) {
                                $model->setData($field["dbcolumnname"], $valStr);
                            }
                        } else {
                            $model->setData($field["dbcolumnname"], $post[$field["postname"]]);
                        }
                    }
                }

                $model->save();
            }
        }
        return $email;
    }

    public function afterSubscribeCustomerById(
    $subscriber, $customerId
    ) {

        if ($this->_helper->getConfig("popup/enabled")) {
            if (!$this->_helper->getPopupCookie()) {
                $this->_helper->setPopupCookie();
            }
        }

        if ($this->_helper->getConfig("general/enabled")) {

            $fieldsMapping = unserialize($this->_helper->getConfig('general/fields_mapping'));

            if ($fieldsMapping && count($fieldsMapping)) {
                $post = $this->request->getPost();


                $model = $this->_subscriberInfoFactory->create();
                $model->setSubscriberId($subscriber->getSubscriberId());

                foreach ($fieldsMapping as $field) {

                    if (isset($post[$field["postname"]])) {
                        if (is_array($post[$field["postname"]])) {
                            $valStr = "";
                            foreach ($post[$field["postname"]] as $val) {
                                if (is_string($val)) {
                                    if ($valStr) {
                                        $valStr .= ' | ' . $val;
                                    } else {
                                        $valStr = $val;
                                    }
                                }
                            }

                            if ($valStr) {
                                $model->setData($field["dbcolumnname"], $valStr);
                            }
                        } else {
                            $model->setData($field["dbcolumnname"], $post[$field["postname"]]);
                        }
                    }
                }

                $model->save();
            }
        }
        return $customerId;
    }

    /**
     * Confirms subscriber newsletter
     *
     * @param string $code
     * @return boolean
     */
    public function afterConfirm($subscriber, $code) {

        if ($this->_helper->getConfig("general/enabled")) {
            if ($subscriber->getCode() == $code) {

                $params = $this->request->getParams();

                if (isset($params["thankyou_id"]) && $params["thankyou_id"] != 0) {
                    $this->_helper->setThankYouCookie($params["thankyou_id"]);
                }

            }
        }
        return $code;
    }

    public function afterGetConfirmationLink($subscriber, $result) {
        if ($this->_helper->getConfig("general/enabled")) {
            $params = $this->request->getParams();

            $fieldsMapping = unserialize($this->_helper->getConfig('general/fields_preconfirm_get'));

            foreach ($params as $key => $value) {

                $addParam = false;

                if ($fieldsMapping && count($fieldsMapping)) {

                    foreach ($fieldsMapping as $field) {

                        if ($key == $field["getname"]) {
                            $addParam = true;
                            break;
                        }
                    }
                }
                
                if ($key == "thankyou_id") {
                    $addParam = true;
                }
                
                if ($addParam) {
                    $result .= $key . "/" . urlencode($value) . "/";
                }
            }
        }

        return $result;
    }

    public function beforeSendConfirmationSuccessEmail(\Magento\Newsletter\Model\Subscriber $subject) {
        if ($this->scopeConfig->getValue('filoblu_newsletter_section/general/disable_newsletter_confirmation_success_email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)) {
            $subject->setImportMode(true);
        }
    }

}
